<?php

/**
 * Clase para generar un formulario de busqueda simple
 * genera el sql relacionado a la busqueda 
 * @package simple_search 
 * @author Gerardo miguel Ballena Descalzo <unmsmgerardo@hotmail.com>
 * @version 1.0
 * @date 01/06/2005
 * @copyright ISPGRAF.SAC - http://www.ispgraf.com
 */
class simple_search {

    var $DB;
    var $multiple_search;
    var $columns;
    var $sql_template;

    function simple_search(&$DB) {
        $this->DB = &$DB;
        $this->multiple_search = false;
        $this->columns = array();
        $this->sql_template = null;
    }

    /**
     * Te dice si se ha activado la busqueda multiple (usa un index FULLTEXT).
     * @return boolean Si es true indica que esta activado la busqueda multiple.
     */
    function get_multiple_search() {
        return $this->multiple_search;
    }

    /**
     * Establece si la busqueda va ser multiple (usa un index FULLTEXT). 
     * @param boolean $multiple_search Valor Booleano que indica si se activa la busqueda multiple.
     * @return boolean True si se establece el valor. 
     */
    function set_multiple_search($multiple_search) {
        $ok = false;

        if (is_bool($multiple_search)) {
            $this->multiple_search = $multiple_search;
            $ok = true;
        }

        return $ok;
    }

    /**
     * Añade una columna de busqueda.
     * @param string $name Rotulo que aparecera en la lista desplegable.
     * @param string $name_column Nombre de la columna en la base de datos.
     * @param string $tables Nombre de la tabla al que pertenece la columna.
     * @return boolean siempre devuelve true  
     */
    function add_column_search($name, $name_column, $table = null) {
        $ok = true;

        $column = new column_search();

        $column->name = $name;
        $column->name_column = $name_column;

        if (!is_null($table)) {
            $column->type_column = 0;
            $sql = " SELECT " . $name_column . " FROM " . $table . " LIMIT 0, 1 ";
            $result = $this->DB->db_query($sql);
            $column->type_data = mysql_field_type($result, 0);
        } else {
            $column->type_column = 1;
            $column->type_data = "string";
        }

        $column->table = $table;

        array_push($this->columns, $column);

        return $ok;
    }

    /**
     * @access private
     * Busca una columna de busqueda.
     * @param string $name Nombre de la columna.
     * @return integer Indice dentro del arreglo $this->columns.  
     */
    function search_column($name) {
        $index = null;

        $i = 0;
        $salir = false;
        while ($i < count($this->columns) and ! $salir) {
            if ($this->columns[$i]->name == $name) {
                $index = $i;
                $salir = true;
            }

            $i++;
        }

        return $index;
    }

    /**
     * Devuelve la plantilla del sql.
     * @return string Cadena conteniendo la plantilla del sql.
     */
    function get_sql_template() {
        return $this->sql_template;
    }

    /**
     * Establece la plantilla del valor.
     * @param string $name Nombre de la columna.
     * @param string $value_template Plantilla del valor.
     * @return boolean True si se establece el valor.
     */
    function set_value_template($name, $value_template) {
        $ok = false;

        $index = $this->search_column($name);

        if (!is_null($index)) {
            $this->columns[$index]->value_template = $value_template;
            $ok = true;
        }

        return $ok;
    }

    /**
     * Devuelve si existe la clausula WHERE en la sentencia sql.
     * @param string $sql Sentencia sql.
     * @return boolean True si existe la clausula.
     */
    function exist_where($sql) {
        $exist = false;

        if (strpos($sql, "where") !== false) {
            $exist = true;
        } elseif (strpos($sql, "WHERE") !== false) {
            $exist = true;
        }

        return $exist;
    }

    /**
     * Devuelve si existe la clausula HAVING en la sentencia sql.
     * @param string $sql Sentencia sql.
     * @return boolean True si existe la clausula.
     */
    function exist_having($sql) {
        $exist = false;

        if (strpos($sql, "having") !== false) {
            $exist = true;
        } elseif (strpos($sql, "HAVING") !== false) {
            $exist = true;
        }

        return $exist;
    }

    /**
     * @access private  
     * Reemplaza los parametros en la plantilla del sql y la plantilla del valor.
     * @param string $value Valor a reemplazar.
     * @return string Retorna el sql con los valores reemplazados.
     */
    function replace_param($value) {
        global $_GET;

        if (!$this->multiple_search) {
            $column = $this->columns[$_GET["column_search"]];

            if ($column->type_column == 0) {
                $exist_where = $this->exist_where($this->sql_template);
            } else {
                $exist_having = $this->exist_having($this->sql_template);
            }

            if (is_null($column->value_template)) {
                //echo $column->type_data."<br>";
                switch ($column->type_data) {
                    case "int":
                    case "real":
                        $value = " = " . $value . " ";
                        break;
                    case "string":
                    case "text":
                    case "blob":
                        $value = " LIKE '%" . $value . "%' ";
                        break;
                    case "date":
                        $value = " between '" . $value . " 00:00:00' and  '" . $value . " 23:59:59'";
                        break;
                    case "year":
                    case "time":
                    case "datetime":
                    case "timestamp":
                        $value = " = '" . $value . "' ";
                        break;
                }
            } else {
                $value = str_replace("[value]", $value, $column->value_template);
            }

            if ($column->type_column == 0) {
                if ($exist_where) {
                    $value_param = " AND " . $column->table . "." . $column->name_column . " ";
                } else {
                    $value_param = " WHERE " . $column->table . "." . $column->name_column . " ";
                }

                $value_param .= $value . " ";

                $sql = str_replace("[simple_search_w]", $value_param, $this->sql_template);
                $sql = str_replace("[simple_search_h]", "", $sql);
            } else {
                if ($exist_having) {
                    $value_param = " AND " . $column->name_column . " ";
                } else {
                    $value_param = " HAVING " . $column->name_column . " ";
                }

                $value_param .= $value . " ";

                $sql = str_replace("[simple_search_w]", "", $this->sql_template);
                $sql = str_replace("[simple_search_h]", $value_param, $sql);
            }
        } else {
            $value_param = "";

            if (trim($value) != "") {
                $exist_where = $this->exist_where($this->sql_template);

                if ($exist_where) {
                    $value_param = " AND ";
                } else {
                    $value_param = " WHERE ";
                }

                $value_param .= " MATCH ( ";

                for ($i = 0; $i < count($this->columns); $i++) {
                    if ($i != 0) {
                        $value_param .= " , ";
                    }

                    $value_param .= $this->columns[$i]->table . "." . $this->columns[$i]->name_column;
                }

                $value_param .= " ) AGAINST ('" . $value . "') ";
            }

            $sql = str_replace("[simple_search_w]", $value_param, $this->sql_template);
            $sql = str_replace("[simple_search_h]", "", $sql);
        }

        return $sql;
    }

    /**
     * Devuelve el sql con los valores de los parametros reemplazados.
     * @param string $sql Plantilla del sql.
     * @return boolean Retorna el sql con los valores reemplazados.
     */
    function get_sql(&$sql) {
        global $_GET;

        $ok = true;

        $this->sql_template = $sql;

        if (isset($_GET["txt_in"]) && $_GET["txt_in"] != "") {
            $sql = $this->replace_param($_GET["txt_in"]);
        } else {
            $sql = str_replace("[simple_search_w]", "", $this->sql_template);
            $sql = str_replace("[simple_search_h]", "", $sql);
        }

        return $ok;
    }

    /**
     * Devuelve el formulario de busqueda en una cadena.
     */
    function get_str() {

        global $_GET;

        $str = "
	<div class='busqueda'>
	
	<form name='form1' method='get'>";
        $var_url = "";

        $raunonly_tin = "";



        if (count($_GET) > 0) {
            foreach ($_GET as $key => $value) {
                if ($key != "nr") {
                    $var_url .= $key . "=" . $value . "&";
                }

                if ($key != "column_search" and $key != "txt_in" and $key != "btn_busc") {
                    $str .= "<input type='hidden' name='" . $key . "' value='" . $value . "'>";
                }
            }
        }

        $str .= "";
        $str .= "<label>Búsqueda :</label>";

        if (!$this->multiple_search) {
            $str .= "<select name='column_search' >";

            if (!isset($_GET["column_search"])) {
                $_GET["column_search"] = 0;
            }

            for ($i = 0; $i < count($this->columns); $i++) {
                $str .= "<option value='" . $i . "' ";

                if ($i == $_GET["column_search"]) {
                    $str .= " SELECTED ";
                }

                $str .= ">" . $this->columns[$i]->name . "</option> ";
            }

            $str .= "</select> ";
        }

        $str .= "<input type='text' " . $raunonly_tin . "  class='text medium-input' name='txt_in' value='";

        if (isset($_GET["txt_in"]))
            $str .= $_GET["txt_in"];
        $str .= "'  />";



        $str .= "
	<button title='Buscar' name='btn_busc' id='btn_busc' class='buscar_btn_ui' type='submit'>Buscar</button>
	";
        $str .= "</form>
	
	
	</div>";
        if (isset($_GET["excel"]) && $_GET["excel"] == "1") {
            return "";
        } else {
            return $str;
        }
    }

    /**
     * Muestra el formulario de busqueda.
     */
    function show() {
        echo $this->get_str();
    }

}

//Fin clase search

class column_search {

    var $name;
    var $name_column;
    var $type_column; //0 -> Columna normal, 1 -> Columna calculada 
    var $type_data;
    var $table;
    var $value_template;

    function column_search() {
        $this->name = null;
        $this->name_column = null;
        $this->type_data = null;
        $this->table = null;
        $this->value_template = null;
    }

}

//Fin clase column_search
?>